;   This file is a basic code template for assembly code generation   *
;   on the PICmicro PIC16F819. This file contains the basic code      *
;   building blocks to build upon.                                    *
;                                                                     *
;   If interrupts are not used all code presented between the ORG     *
;   0x004 directive and the label main can be removed. In addition    *
;   the variable assignments for 'w_temp' and 'status_temp' can       *
;   be removed.                                                       *
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler (Document DS33014).                     *
;                                                                     *
;   Refer to the respective PICmicro data sheet for additional        *
;   information on the instruction set.                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    AKA_ZU.asm                                           *
;    Date:                                                            *
;    File Version:                                                    *
;                                                                     *
;    Author:                                                          *
;    Company:                                                         *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                         *
;            KASIL                                       *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************

	list      p=16f819           ; list directive to define processor
	#include <p16F819.inc>        ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file

	__CONFIG   _CP_OFF & _WRT_ENABLE_OFF & _CPD_OFF & _CCP1_RB2 & _DEBUG_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_IO 

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.




;***** VARIABLE DEFINITIONS
w_temp        	EQU     0x70        ; variable used for context saving 
status_temp   	EQU     0x71        ; variable used for context saving
pclath_temp	  	EQU		0x72	

	cblock 0x30
CCP_temp
CCP2_temp
I: 0, I_H, I_L					;   	
V: 0, V_H, V_L					;  
VT: 0, VT_H, VT_L
	endc




;**********************************************************************
	ORG     0x000             ; processor reset vector
	goto    main              ; go to beginning of program
	

	ORG     0x004             ; interrupt vector location
	movwf   w_temp            ; save off current W register contents
	movf	STATUS,w          ; move status register into W register
	movwf	status_temp       ; save off contents of STATUS register
	MOVF    PCLATH, w
	MOVWF   pclath_temp
	CLRF    PCLATH
;===========    !!! =============	
    MOVF    pclath_temp, w
    MOVWF   PCLATH
	movf    status_temp,w     ; retrieve copy of STATUS register
	movwf	STATUS            ; restore pre-isr STATUS register contents
	swapf   w_temp,f
	swapf   w_temp,w          ; restore pre-isr W register contents
	retfie                    ; return from interrupt


main
    CLRF    STATUS          	; Do initialization (Bank 0)

	;     ,     .
    BSF     STATUS, RP0     	; Select Bank 1
	MOVLW	b'01110100'			;      8 
	MOVWF	OSCCON
    BCF     STATUS, RP0     	; Select Bank 0

	CLRF	PORTA				;    A -  "0"
	CLRF	PORTB				;    B -  "0"

    BSF     STATUS, RP0     	; Select Bank 1
    MOVLW   b'00101011'
    MOVWF   TRISA           	;  A -  ,  RA2, RA4, RA6, RA7.
    MOVLW   b'00000000'
    MOVWF   TRISB           	;  B -   .
    BSF     OPTION_REG,NOT_RBPU	;     B   
    BCF     STATUS, RP0     	; Select Bank 0

; -------    ----------
	CLRF	TMR2
	CLRF	T2CON				;    TMR2  1:1,  
	BSF		STATUS, RP0			;    1
	MOVLW	.59					;   (  33 )
	MOVWF	PR2					;       
	BCF		STATUS, RP0			;    0

	MOVLW	.40
	MOVWF	CCPR1L				;      50%
	MOVLW	0x0C			
	MOVWF	CCP1CON				;   CCP2/PWM2   . 
; -------------------------------------------
; --------   ----------------
    BSF     STATUS, RP0     	; Select Bank 1
    MOVLW   b'11000100'     	;  , T=32*Tosc, ... 
    MOVWF   ADCON1          	; ...   AN0, AN1  AN3,   - Vss  Vdd
    BCF     STATUS, RP0     	; Select Bank 0
    MOVLW   b'01000001'     	; T=16*Tosc,   - AN0,   , ...
    MOVWF   ADCON0          	; ...  .
; -------------------------------------------

	BSF		T2CON, TMR2ON		;  

loop:
;---------------------    -------------------
    MOVLW   b'01001001'     	; T=16*Tosc,   - AN0,   , ...
    MOVWF   ADCON0          	; ...  .
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	BSF		ADCON0, GO			;    
ADC_I_Loop:
	BTFSC	ADCON0, GO			;      
	GOTO	ADC_I_Loop
    BSF     STATUS, RP0     	; Select Bank 1	(   -   1 !!!)
	MOVFW	ADRESL				;     , ..     1 
    BCF     STATUS, RP0     	; Select Bank 0
	MOVWF	I_H
;---------------------------------------------------------------

	BZ		no_curr
	BSF		PORTA, 2			;       
	GOTO	end_curr_ind
no_curr:
	BCF		PORTA, 2			;       
end_curr_ind:

;---------------------    -------------------
    MOVLW   b'01000001'     	; T=16*Tosc,   - AN0,   , ...
    MOVWF   ADCON0          	; ...  .
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	BSF		ADCON0, GO			;    
ADC_V_Loop:
	BTFSC	ADCON0, GO			;      
	GOTO	ADC_V_Loop
	MOVFW	ADRESH
	MOVWF	V_H
    BSF     STATUS, RP0     	; Select Bank 1	(   -   1 !!!)
	MOVFW	ADRESL				
    BCF     STATUS, RP0     	; Select Bank 0
	MOVWF	V_L
;---------------------------------------------------------------------------
; -------------     ------------
	RRF		V_H, f
	RRF		V_L, f
	CLRC
	RRF		V_H, f
	RRF		V_L, f
	CLRC

	MOVFW	V_L
	MOVWF	VT_H
; ----------------------------------------------------------	
;-----   -----
;  10,5 V
	MOVFW	VT_H
	SUBLW	.106
	BNC		_rb0_on
	BCF		PORTB, 3
	GOTO	_end_rb0
_rb0_on:
	BSF		PORTB, 3
_end_rb0:

;  11,0 V
	MOVFW	VT_H
	SUBLW	.111
	BNC		_rb1_on
	BCF		PORTB, 1
	GOTO	_end_rb1
_rb1_on:
	BSF		PORTB, 1
_end_rb1:

;  11,5 V
	MOVFW	VT_H
	SUBLW	.116
	BNC		_rb3_on
	BCF		PORTB, 0
	GOTO	_end_rb3
_rb3_on:
	BSF		PORTB, 0
_end_rb3:

;  12,0 V
	MOVFW	VT_H
	SUBLW	.121
	BNC		_rb4_on
	BCF		PORTB, 4
	GOTO	_end_rb4
_rb4_on:
	BSF		PORTB, 4
_end_rb4:

;  12,5 V
	MOVFW	VT_H
	SUBLW	.126
	BNC		_rb5_on
	BCF		PORTB, 5
	GOTO	_end_rb5
_rb5_on:
	BSF		PORTB, 5
_end_rb5:

;  13,0 V
	MOVFW	VT_H
	SUBLW	.131
	BNC		_rb6_on
	BCF		PORTB, 6
	GOTO	_end_rb6
_rb6_on:
	BSF		PORTB, 6
_end_rb6:

;  13,5 V
	MOVFW	VT_H
	SUBLW	.136
	BNC		_rb7_on
	BCF		PORTB, 7
	GOTO	_end_rb7
_rb7_on:
	BSF		PORTB, 7
_end_rb7:

;  14,0 V
	MOVFW	VT_H
	SUBLW	.141
	BNC		_ra6_on
	BCF		PORTA, 6
	GOTO	_end_ra6
_ra6_on:
	BSF		PORTA, 6
_end_ra6:

;  14,5 V
	MOVFW	VT_H
	SUBLW	.146
	BNC		_ra7_on
	BCF		PORTA, 7
	GOTO	_end_ra7
_ra7_on:
	BSF		PORTA, 7
_end_ra7:

;  15,0 V
	MOVFW	VT_H
	SUBLW	.151
	BNC		_ra4_on
	BCF		PORTA, 4
	GOTO	_end_ra4
_ra4_on:
	BSF		PORTA, 4
_end_ra4:
;----------------------------------
; -----       -----
	MOVFW	CCPR1L			;      ...
	MOVWF	CCP_temp		; ...   
	CLRC
	RLF		CCP_temp, f		;       
	CLRC					;      
	RLF		CCP_temp, f
	CLRC					;      
	
	MOVFW	CCP1CON
	ANDLW	b'00110000'		;     5  4 
	MOVWF	CCP2_temp
	SWAPF	CCP2_temp, w	;   5  4   1  0
	
	IORWF	CCP_temp, f		;     
; ---------------------------------------------------------

; -----    ------
	MOVFW	VT_H
	SUBLW	.148			; ,     14,7 V
	BC		limit_i			;   14,7    -     

	TSTF	CCP_temp
	SKPZ
	DECF	CCP_temp, f
	GOTO	end_analise
	
; -----    -----
limit_i:
	MOVFW	I_H
	SUBLW	.48				;         0,2 A
	BC		i_up			;     -   
		
; -----     () ------
	TSTF	CCP_temp		
	SKPZ
	DECF	CCP_temp, f
	GOTO	end_analise

; -----     () ------
i_up:
	INCFSZ	CCP_temp, w		;    ...
	MOVWF	CCP_temp		; ...        FF

end_analise:
; ---------------------------------------------------------------------

; -----        
	MOVFW	CCP_temp
	ANDLW	b'00000011'		;    
	MOVWF	CCP2_temp
	SWAPF	CCP2_temp, f	;        CCP1CON 
	MOVFW	CCP1CON

	ANDLW	b'11001111'		;      

	IORWF	CCP2_temp, f	;    CCP1CON    CCP2_temp
	
	CLRC
	RRF		CCP_temp, f
	CLRC
	RRF		CCP_temp, f
	CLRC					;    CCPR1L    CCP_temp
	
	MOVFW	CCP_temp
	MOVWF	CCPR1L
	MOVFW	CCP2_temp
	MOVWF	CCP1CON
; ----------------------------------------------------------------------	

end_PWM:
	GOTO	loop	

	END                       ; directive 'end of program'

